import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;

import java.io.*;
import java.math.BigDecimal;
import java.net.URL;
import java.util.*;

/**
 * Created by Mark Hillman on 12/07/2016.
 */
public class Controller implements Initializable {

    @FXML Button generate;
    @FXML Spinner<Integer> passwordLength;
    @FXML TextField password;
    @FXML TextArea characters;
    @FXML ChoiceBox styles;
    @FXML Label entropy;
    @FXML Label status;

    Map<String, String> styleMap;

    @Override
    public void initialize(URL location, ResourceBundle resources) {

        // Read the options from the file
        String optionsString = readFile("options.properties");
        if (optionsString == null)
            optionsString = "Alphabet=abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
        String[] options = optionsString.split("\n");
        styleMap = new TreeMap<>(); // TreeMap for alphabetical
        for (int i = 0; i < options.length; i++) {
            String[] keyValue = options[i].split("=");
            styleMap.put(keyValue[0], keyValue[1]);
        }

        // Set the options for style
        List<String> keys = new ArrayList<>();
        keys.addAll(styleMap.keySet());
        ObservableList<String> observableList = FXCollections.observableList(keys);
        styles.setItems(observableList);
        styles.setValue(styles.getItems().get(0));

        // Set the characters when the password style is changed
        styles.setOnAction(event -> {
            characters.setText(styleMap.get(styles.getValue()));
            setStatus(styles.getValue() + " chosen.");
            calculateEntrophy();
        });
        styles.fireEvent(new ActionEvent());
        characters.textProperty().addListener((observable, oldValue, newValue) -> calculateEntrophy());
        passwordLength.valueProperty().addListener((observable, oldValue, newValue) -> calculateEntrophy());
        passwordLength.setEditable(true);

        calculateEntrophy();
    }

    // Read the file line by line
    private String readFile(String filename) {
        try {
            File file = new File(filename);
            FileInputStream fis = new FileInputStream(file);
            byte[] data = new byte[(int)file.length()];
            fis.read(data);
            fis.close();
            return new String(data, "UTF-8");
        } catch (FileNotFoundException e) {
            System.out.println("Error: The file could not be found");
        } catch (IOException e) {
            System.out.println("Error: IOException happened");
        }
        return null;
    }

    // Generate the password and add it to password field
    public void generatePassword() {
        String chars = characters.getText();
        int length = passwordLength.getValue();
        if (chars.length() > 0) {
            String passwordString = makePassword(chars, length);
            password.setText(passwordString);
            setStatus("Password generated!");
        }
        else {
            setStatus("Enter characters for password!");
        }
    }

    // Make the password from the characters supplied for the given length
    private String makePassword(String chars, int length) {
        String password = "";
        for (int i = 0; i < length; i++) {
            int rand = (int)Math.floor(Math.random() * chars.length());
            password += chars.charAt(rand);
        }
        return password;
    }

    private void setStatus(String statusString) {
        status.setText("Status: " + statusString);
    }

    private void calculateEntrophy() {
        int length = passwordLength.getValue();
        int charsetLength = characters.getLength();
        double entropyValue = Math.pow(charsetLength, length);
        BigDecimal bd = new BigDecimal(entropyValue); // Stops 0 truncations
        String format = "Entropy: %ef";
        if (bd.toString().length() < 35)
            format = "Entropy: %.0f";
        entropy.setText(String.format(format, entropyValue));
    }
}
